from TEM_comms.ui import Run
from pydantic import ValidationError
import pytest


def test_run():
    with pytest.raises(ValidationError):
        Run(montage=True, session_id="test", grid_first=1)

    with pytest.raises(ValidationError):
        Run(montage=True, session_id="test", grid_last=2)

    with pytest.raises(ValidationError):
        Run(montage=True, grid_first=1, grid_last=2)

    Run(montage=True, session_id="test", grid_first=1, grid_last=2)
    msg = Run()
    assert not msg.montage



from pigeon import BaseMessage


class Focus(BaseMessage):
    montage_id: str
    tile_id: str
    focus: float


class Histogram(BaseMessage):
    montage_id: str
    tile_id: str
    path: str


class MinMaxMean(BaseMessage):
    montage_id: str
    tile_id: str
    min: int
    max: int
    mean: int




from . import buffer
from . import camera
from . import scope
from . import stage
from . import tile
from . import ui
from . import montage
from . import qc


topics = {
    "buffer.status": buffer.Status,
    "camera.command": camera.Command,
    "camera.image": camera.Image,
    "camera.settings": camera.Settings,
    "camera.status": camera.Status,
    "scope.command": scope.Command,
    "scope.status": scope.Status,
    "stage.aperture.command": stage.aperture.Command,
    "stage.aperture.status": stage.aperture.Status,
    "stage.motion.command": stage.motion.Command,
    "stage.motion.status": stage.motion.Status,
    "stage.rotation.command": stage.rotation.Command,
    "stage.rotation.status": stage.rotation.Status,
    "tile.jpeg": tile.JPEG,
    "tile.minimap": tile.Minimap,
    "tile.processed": tile.Processed,
    "tile.raw": tile.Raw,
    "tile.statistics.focus": tile.statistics.Focus,
    "tile.statistics.histogram": tile.statistics.Histogram,
    "tile.statistics.min_max_mean": tile.statistics.MinMaxMean,
    "tile.transform": tile.Transform,
    "ui.edit": ui.Edit,
    "ui.run": ui.Run,
    "ui.setup": ui.Setup,
    "montage.start": montage.Start,
    "montage.finished": montage.Finished,
    "qc.status": qc.Status,
}



from pigeon import BaseMessage
from . import statistics


class JPEG(BaseMessage):
    tile_id: str
    path: str


class Minimap(BaseMessage):
    tile_id: str
    path: str


class Raw(BaseMessage):
    tile_id: str
    montage_id: str
    path: str
    row: int
    column: int
    overlap: int


class Transform(BaseMessage):
    montage_id: str
    tile_id: str
    rotation: float
    x: float
    y: float


class Processed(BaseMessage):
    montage_id: str
    tile_id: str
    path: str



from pigeon import BaseMessage
from typing import Mapping, List, Any


class Start(BaseMessage):
    montage_id: str
    num_tiles: int


class Finished(BaseMessage):
    montage_id: str
    num_tiles: int
    roi: str
    specimen: str
    metadata: Mapping[str, Any] | List[Any]



from pigeon import BaseMessage
from typing import Optional
from pydantic import model_validator


class Edit(BaseMessage):
    roi_id: str
    roi_pos_x: int
    roi_pox_y: int
    roi_width: int
    roi_height: int
    roi_angle: float


class Run(BaseMessage):
    session_id: Optional[str] = None
    grid_first: Optional[int] = None
    grid_last: Optional[int] = None
    montage: bool = False
    abort_now: bool = False
    abort_at_end: bool = False
    resume: bool = False

    @model_validator(mode="after")
    def check_grid(self):
        assert self.montage != (self.grid_first is None)
        assert self.montage != (self.grid_last is None)
        return self

    @model_validator(mode="after")
    def check_session(self):
        assert self.montage != (self.session_id is None)
        return self


class Setup(BaseMessage):
    conch_owner: Optional[str] = None
    auto_focus: bool = False
    auto_exposure: bool = False
    lens_correction: bool = False
    acquire_brightfield: bool = False
    acquire_darkfield: bool = False



from pigeon import BaseMessage
from typing import Literal, Optional, Tuple
from pydantic import model_validator


class Command(BaseMessage):
    focus: Optional[int] = None
    mag_mode: Optional[Literal["LM", "MAG1", "MAG2"]] = None
    mag: Optional[int] = None
    brightness: Optional[int] = None
    beam_offset: Optional[Tuple[int, int]] = None
    screen: Optional[Literal["up", "down"]] = None

    @model_validator(mode="after")
    def check_mag(self):
        assert (self.mag_mode is None) == (self.mag is None)
        return self


class Status(BaseMessage):
    focus: int
    aperture: str | None
    mag_mode: str
    mag: int
    tank_voltage: int
    brightness: int
    beam_offset: Tuple[int, int]



import re

import pytest
from TEM_comms.buffer import Status

from pydantic import ValidationError


# Test cases for the Status class
@pytest.mark.parametrize(
    "queue_length, free_space, upload_rate, test_id",
    [
        (10, 100, 20, "test_id_01"),  # Happy path: normal values
        (0, 0, 0, "test_id_02"),  # Edge case: zero values
        (100, 500, 100, "test_id_03"),  # Happy path: higher values
        (-1, -100, -10, "test_id_04"),  # Error case: negative values
        (1e6, 1e6, 1e4, "test_id_05"),  # Edge case: very large values
    ],
)
def test_status_initialization(queue_length, free_space, upload_rate, test_id):
    # Act
    status = Status(
        queue_length=queue_length, free_space=free_space, upload_rate=upload_rate
    )

    # Assert
    assert (
        status.queue_length == queue_length
    ), f"{test_id}: queue_length does not match"
    assert status.free_space == free_space, f"{test_id}: free_space does not match"
    assert status.upload_rate == upload_rate, f"{test_id}: upload_rate does not match"


# Test cases for handling type errors
@pytest.mark.parametrize(
    "queue_length, free_space, upload_rate, test_id",
    [
        (
            "ten",
            100,
            20,
            "test_id_06",
        ),  # Error case: string instead of int for queue_length
        (
            10,
            "hundred",
            20,
            "test_id_07",
        ),  # Error case: string instead of int for free_space
        (
            10,
            100,
            "twenty",
            "test_id_08",
        ),  # Error case: string instead of int for upload_rate
    ],
)
def test_status_type_errors(queue_length, free_space, upload_rate, test_id):
    # Act and Assert
    expected_error_pattern = re.escape("validation error")
    with pytest.raises(ValidationError, match=expected_error_pattern):
        Status(
            queue_length=queue_length, free_space=free_space, upload_rate=upload_rate
        )



from pigeon import BaseMessage


class Command(BaseMessage):
    angle_x: float | None = None
    angle_y: float | None = None
    calibrate: bool = False


class Status(BaseMessage):
    angle_x: float
    angle_y: float
    in_motion: bool
    error: str = ""



from TEM_comms.scope import Command
from pydantic import ValidationError
import pytest


def test_mag():
    with pytest.raises(ValidationError):
        Command(mag_mode="LM")

    with pytest.raises(ValidationError):
        Command(mag=10)

    Command()
    Command(mag_mode="LM", mag=1)


def test_mag_mode():
    with pytest.raises(ValidationError):
        Command(mag_mode="test", mag=1)

    Command(mag_mode="LM", mag=1)
    Command(mag_mode="MAG2", mag=1)
    Command(mag_mode="MAG2", mag=1)



from pigeon import BaseMessage


class Command(BaseMessage):
    aperture_id: int | None = None
    calibrate: bool = False


class Status(BaseMessage):
    current_aperture: int
    calibrated: bool
    error: str = ""



from pigeon import BaseMessage


class Command(BaseMessage):
    x: int | None = None
    y: int | None = None
    calibrate: bool = False


class Status(BaseMessage):
    x: int
    y: int
    in_motion: bool
    error: str = ""



from pigeon import BaseMessage
from typing import Optional


class Command(BaseMessage):
    tile_id: str


class Image(BaseMessage):
    tile_id: str
    path: str


class Settings(BaseMessage):
    exposure: float | None = None
    gain: Optional[float] = None
    width: int | None = None
    height: int | None = None


class Status(BaseMessage):
    exposure: float
    gain: float
    width: int
    height: int
    temp: float
    target_temp: float
    device_name: str
    device_model_id: int
    device_sn: str
